package com.ruoyi.ss.bonus.service.impl;

import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.bonus.domain.Bonus;
import com.ruoyi.ss.bonus.domain.enums.BonusArrivalType;
import com.ruoyi.ss.bonus.domain.enums.BonusStatus;
import com.ruoyi.ss.bonus.service.BonusConverter;
import com.ruoyi.ss.device.domain.enums.DeviceServiceMode;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceAssembler;
import com.ruoyi.ss.store.service.StoreService;
import com.ruoyi.ss.transactionBill.domain.TransactionBill;
import com.ruoyi.ss.transactionBill.domain.bo.RechargeBO;
import com.ruoyi.ss.transactionBill.domain.dto.RechargePayBO;
import com.ruoyi.ss.transactionBill.domain.vo.TransactionBillVO;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.user.domain.SmUserVO;
import com.ruoyi.ss.user.service.ISmUserService;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @author wjh
 * 2024/8/20
 */
@Service
public class BonusConverterImpl implements BonusConverter {

    @Autowired
    private ISmUserService userService;

    @Autowired
    private StoreService storeService;

    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private ISysConfigService sysConfigService;

    @Autowired
    private DeviceAssembler deviceAssembler;

    /**
     * 订单转为分成明细
     *
     * @param bo
     */
    @Override
    public List<Bonus> toPo(RechargePayBO bo) {
        if (bo == null) {
            return Collections.emptyList();
        }

        TransactionBill bill = bo.getOrder();
        if (bill == null) {
            return Collections.emptyList();
        }

        List<Bonus> result = genBonusList(bo.getMch(), null, bo.getPlatform(), bo.getDevice());

        for (Bonus bonus : result) {
            bonus.setBillId(bill.getBillId());
            bonus.setBillNo(bill.getBillNo());
        }

        return result;
    }

    @Override
    public List<Bonus> toPo(RechargeBO bo) {
        if (bo == null) {
            return Collections.emptyList();
        }

        TransactionBill bill = bo.getOrder();
        if (bill == null) {
            return Collections.emptyList();
        }

        return genBonusList(bo.getMch(), bo.getAgent(), bo.getPlatform(), bo.getDevice());
    }

    @Override
    public List<Bonus> genBonusList(SmUserVO mch, SmUserVO agent, SysDept platform, DeviceVO device) {
        List<Bonus> result = new ArrayList<>();
        if (mch == null || platform == null || device == null) {
            return result;
        }
        // 剩余可分配的分成比例
        BigDecimal point = BigDecimal.valueOf(100);

        // 直营模式
        if (DeviceServiceMode.DIRECT.getMode().equals(device.getServiceMode())) {
            // 拼接设备服务费费率
            deviceAssembler.assembleRealServiceRate(device);

            // 按照百分比收取服务费
            BigDecimal serviceRate = device.getRealServiceRate();   // 服务费比例
            result.add(this.toPo(platform, serviceRate));

            point = point.subtract(serviceRate);
        }
        // 代理模式
        else if (DeviceServiceMode.AGENT.getMode().equals(device.getServiceMode())) {
            // 平台收取服务费
            ServiceUtil.assertion(agent == null, "代理商不存在，请联系管理员处理");
            ServiceUtil.assertion(agent.getAgentServiceRate() == null, "代理商服务费未配置");
            ServiceUtil.assertion(device.getAgentServiceRate() == null, "代理商设备服务费未配置");

            result.add(this.toPo(platform, agent.getAgentServiceRate()));
            point = point.subtract(agent.getAgentServiceRate());

            // 代理商收取服务费
            BigDecimal agentArrivalPoint = device.getAgentServiceRate().subtract(agent.getAgentServiceRate());
            result.add(this.toPo(agent, agentArrivalPoint, agent.getUserId().toString(), BonusArrivalType.AGENT));
            point = point.subtract(agentArrivalPoint);
        }

        ServiceUtil.assertion(point.compareTo(BigDecimal.ZERO) < 0, "商户剩余分成不允许小于0");
        ServiceUtil.assertion(point.compareTo(BigDecimal.valueOf(100)) > 0, "商户剩余分成不允许大于100");

        // 剩余的给商户
        String ancestors = Stream.of(device.getAgentId(), device.getUserId())
                .filter(Objects::nonNull)
                .map(Object::toString)
                .collect(Collectors.joining(","));
        result.add(this.toPo(mch, point, ancestors, BonusArrivalType.MCH));

        return result;
    }

    @Override
    public List<Bonus> toPo(TransactionBillVO bill) {
        if (bill == null) {
            return Collections.emptyList();
        }
        LocalDateTime payTime = DateUtils.toLocalDateTime(bill.getPayTime());

        List<Bonus> result = new ArrayList<>();
        BigDecimal decimal100 = new BigDecimal("100");

        // TODO 平台
        // 平台百分比
        BigDecimal platformPoint = bill.getServiceCharge().multiply(decimal100).divide(bill.getMoney(), 2, RoundingMode.HALF_UP);
        Bonus platformBonus = new Bonus();
        platformBonus.setBillId(bill.getBillId());
        platformBonus.setBillNo(bill.getBillNo());
        platformBonus.setStatus(BonusStatus.DIVIDEND.getStatus());
        platformBonus.setArrivalId(Constants.ROOT_DEPT);
        platformBonus.setArrivalName("创特科技");
        platformBonus.setArrivalType(BonusArrivalType.PLATFORM.getType());
        platformBonus.setPoint(platformPoint);
        platformBonus.setAmount(bill.getServiceCharge());
        platformBonus.setRefundAmount(bill.getRefundServiceAmount());
        platformBonus.setAncestors("");
        platformBonus.setDeptId(Constants.ROOT_DEPT);
        platformBonus.setPayTime(payTime);
        platformBonus.setPrePayTime(payTime);
        platformBonus.setPayedAmount(bill.getServiceCharge().subtract(bill.getRefundServiceAmount()));
        platformBonus.setWaitAmount(BigDecimal.ZERO);
        result.add(platformBonus);

        // TODO 商户
        // 商户百分比
        BigDecimal mchPoint = decimal100.subtract(platformPoint);
        Bonus mchBonus = new Bonus();
        mchBonus.setBillId(bill.getBillId());
        mchBonus.setBillNo(bill.getBillNo());
        mchBonus.setStatus(BonusStatus.DIVIDEND.getStatus());
        mchBonus.setArrivalId(bill.getMchId());
        mchBonus.setArrivalName(bill.getMchName());
        mchBonus.setArrivalType(BonusArrivalType.MCH.getType());
        mchBonus.setPoint(mchPoint);
        mchBonus.setAmount(bill.getArrivalAmount());
        mchBonus.setRefundAmount(bill.getRefundMchAmount());
        mchBonus.setAncestors(bill.getMchId().toString());
        mchBonus.setDeptId(Constants.ROOT_DEPT);
        mchBonus.setPayTime(payTime);
        mchBonus.setPrePayTime(payTime);
        mchBonus.setPayedAmount(bill.getArrivalAmount().subtract(bill.getRefundMchAmount()));
        mchBonus.setWaitAmount(BigDecimal.ZERO);
        result.add(mchBonus);

        return result;
    }

    private Bonus toPo(SysDept dept, BigDecimal point) {
        if (dept == null) {
            return null;
        }

        Bonus po = new Bonus();
        po.setStatus(BonusStatus.UN_DIVIDEND.getStatus());
        po.setArrivalId(dept.getDeptId());
        po.setArrivalName(dept.getDeptName());
        BonusArrivalType arrivalType = BonusArrivalType.parseByDept(dept);
        if (arrivalType != null) {
            po.setArrivalType(arrivalType.getType());
        }
        po.setPoint(point);
        po.setAncestors(dept.getAncestors());
        po.setDeptId(dept.getDeptId());

        return po;
    }

    private Bonus toPo(SmUserVO user, BigDecimal point, String ancestors, BonusArrivalType arrivalType) {
        if (user == null || point == null) {
            return null;
        }

        Bonus po = new Bonus();
        po.setStatus(BonusStatus.UN_DIVIDEND.getStatus());
        po.setArrivalId(user.getUserId());
        po.setArrivalName(user.getRealOrUserName());
        po.setArrivalType(arrivalType.getType());
        po.setPoint(point);
        po.setAncestors(ancestors);
        po.setDeptId(Constants.ROOT_DEPT);

        return po;
    }
}
